#!/usr/bin/env python
# coding=utf-8
# 
#	Ferestre principala/login program jeopardy
# (c) Alex Eftimie

import gtk
import network
import sys, os.path, time
from interface import GridView

class MainWindow:
	def on_window1_destroy(self, widget, data=None): 
		gtk.main_quit()
		if self.connector != None:
			self.connector.close()
		if self.udplistener != None:
			self.udplistener.close()		
		sys.exit(0)

	def on_aboutmenu_activate(self, event):
		ad = gtk.AboutDialog()
		ad.set_name("Jeopardy Client")
		ad.set_authors(["Alex Eftimie <alex@eftimie.ro>"])
		ad.set_comments("Client for a jeopardy game\nlistening on %d" % network.PORT)
		ad.set_version('0.3-2')
		ad.set_website('http://elf.cs.pub.ro/uso/jeopardy/')
		ad.run()
		ad.destroy()

	def set_statusbar(self, text):
		sb = self.builder.get_object("statuslabel")
		sb.set_text(str(text))

	def login_window(self):
		builder = gtk.Builder()
		builder.add_from_file(os.path.dirname(__file__) + '/client.glade')
		loginwindow = builder.get_object('loginwindow')
		builder.connect_signals(self)
		self.host_entry = builder.get_object("hostentry")
		self.name_entry = builder.get_object("nameentry")
		self.udpport_entry = builder.get_object("udpportentry")
		self.error_entry = builder.get_object("errorentry")
		self.host_entry.set_text('%s:%d' % (network.SERVER, network.SERVER_PORT))
		self.udpport_entry.set_text('%d' % network.PORT)
		loginwindow.show()
		self.loginwindow = loginwindow
		
	''' Try to connect '''
	def on_connectbutton_clicked(self, widget, data=None): 
		# Clear any previous error
		self.error_entry.set_text("")
		# Try connecting 
		hoststring = self.host_entry.get_text()
		namestring = self.name_entry.get_text()
		udpport = self.udpport_entry.get_text()
		if udpport == '':
			udpport = str(network.PORT)
		network.PORT = int(udpport)
		# Clean name string
		namestring = ''.join(c for c in namestring if c.isalpha())
		result, error, self.id = self.connector.connect(namestring, hoststring)
		if result == True:
			self.loginwindow.hide()
			self.udplistener = network.NetworkListener()
			self.udplistener.set_window(self)
			self.udplistener.start()
			self.name = namestring
			self.score = 0
			self.set_score()
			self.go_idle("Waiting for the game to begin...")
			self.window.show()
		else:
			self.error_entry.set_text(error)
	
	''' Deconectare dorita '''	
	def on_disconnect(self, widget, data=None): 
		self.window.hide()
		self.connector.close()
		self.udplistener.close()
		self.loginwindow.show()
		
	def set_score(self):
		sl = self.builder.get_object('scorelabel')
		sl.set_text('   %s  %5d' % (self.name.ljust(25), self.score))
	
	''' primeste lista de la q id,scor si o afiseaza
		apeleaza si set_score '''
	def set_top(self, top_data):
		tl = self.builder.get_object('toplabel')
		string = ''
		i = 1
		self.leader = top_data[0]
		# sortez inainte de a afisa
		top_data.sort(lambda y,x: x[2]-y[2])
		for user in top_data:
			id, nume, puncte = user
			string = string + "%2d. %s %5d\n" % (i, nume.ljust(30 - len(nume)), puncte)
			if id == self.id:
				self.score = puncte
			i += 1
		self.set_score()		
		tl.set_text(string)
		pass
		
	''' Intru in stare de asteptare '''
	def go_idle(self, text):
		self.disable_texts()
		self.set_statusbar(text)
		
	''' Game over '''
	def game_over(self):
		self.disable_texts()
		id, nume, puncte = self.leader
		self.set_statusbar('Game over! Winner: %s with %d points' % (nume, puncte))
		
	''' Dezactivez toate campurile de text '''
	def disable_texts(self):
		self.builder.get_object('hbox1').set_sensitive(False)
		
	''' Initializez un joc '''
	def init_game(self, gs):
		self.gridsize = gs
		w, h = gs
		self.matrix = [[False for i in range(0,w)] for j in range(0,h)]
		self.set_statusbar("Waiting for the question")
	
	''' Afisez o intrebare venita si activez butonul 
		Face delay la afisare 3 secunde
	'''
	def set_question(self, gridid, data):
		i, j = gridid
		self.matrix[i][j] = True
		if self.local_questions == True:
			data = self.questions[j][i]
		self.builder.get_object('questionlabel').set_text(data)
		self.set_statusbar("")
		self.draw_grid(self.builder.get_object('drawingarea1'), None)
		# Delay 2 secunde TODO: configurabil 3
		self.builder.get_object('questionlabel').set_sensitive(True)
		time.sleep(0.5)
		self.builder.get_object('hbox1').set_sensitive(True)
		
	''' Apasare de buton Yes '''
	def on_yesbutton_clicked(self, widget, data=None):
		self.go_idle("Waiting for the answer")
		self.connector.send_yes()
		
	''' Inghetare din cauza altuia '''
	def freeze(self):
		self.go_idle("Waiting for an answer")
	
	''' Desenare grila intrebari 
		Apelata	on_exposed pentru drawarea1
	'''
	def draw_grid(self, area, event):
		self.gridview.draw(self, area)
			
	def __init__(self):
		# initializez interfata din xml
		builder = gtk.Builder()
		builder.add_from_file(os.path.dirname(__file__) + '/client.glade')
		self.window = builder.get_object('window1')
		#self.label1 = builder.get_object("label1")
		builder.connect_signals(self)
		# salvez builder pentru viitoare folosiri
		self.builder = builder
		# canal comunicatie server
		self.connector = network.Connector(self)
		self.udplistener = None
		# Afisez fereastra principala
		self.login_window()
		# GridView widget
		self.gridview = GridView()
		# Pana la gamestart...
		self.gridsize = (3, 3) 
		self.matrix = [[False for i in range(0,3)] for i in range(0,3)]
		self.leader = (0, "Unkwnown", 0)
		# Questions
		self.local_questions = False
		self.questions = [[]]		# Use it when self.local_questions = True

class Main:
	def __init__(self):
		w = MainWindow()
		# Pornire bucla gtk
		gtk.gdk.threads_init()
		gtk.gdk.threads_enter()
		gtk.main()
		gtk.gdk.threads_leave()

# Daca este pornit din linie de comanda:
if __name__ == "__main__":
	Main()
	
