#!/usr/bin/env python
# encoding=utf-8
#
# Networking
# (c) Alex Eftimie

import sys, os, time
import socket
import pickle
from threading import Thread, Event

HOST = '0.0.0.0'		#'192.168.1.255'		# use it for UDP broadcasting
SERVER = 'Dijkstra.local'	#'192.168.1.2'		# tcp connection server
SERVER_PORT = 5307		# tcp connection server port
PORT = 53072			# jeopa
BUFFER_SIZE = 100

''' Administreaza o conexiune tcp.
	Conexiunea trebuie sa stea activa
'''
class Connector:
	def __init__(self, window):
		self.sock = None
		self.window = window
		
	def connect(self, username, hoststring):
		parts = hoststring.split(":")
		if len(parts) == 2:
			self.port = int(parts[1])
		else:
			self.port = SERVER_PORT
		self.host = parts[0]
		sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		print "Trying to connect to host...", self.port, self.host
		
		try:	
			sock.connect((self.host, self.port))
		except socket.error, msg:
			print msg
			return (False, "Could not connect", 0)
			
		sock.send("u:%s" % (username))
		data = sock.recv(1024*10)
		parts = data.split(":", 4)
		#print "Received answer ", parts
		self.sock = sock
		# Validare raspuns
		if parts[0] != "l":
			return (False, "Invalid answer", 0)
		if parts[1] != "y":	#TODO: different errors
			return (False, "Name rejected", 0)
		# Am primit y, verific daca am si intrebari
		if len(parts) == 4:
			print "Full"
			self.window.local_questions = True
			self.window.questions = pickle.loads(parts[3])
		return (True, "", int(parts[2]))
	
	''' Trimite un mesaj Yes (apasare de buton) la server) '''
	def send_yes(self):
		self.sock.send("b")
		
	''' Inchide conexiunea '''
	def close(self):
		if self.sock != None:
			self.sock.close()

''' Obiect care asculta pe udp apoi notifica fereastra 
	Trebuie sa fie apelata metoda set_window cu un obiect de tip 
	MainWindow in prealabil
'''
class NetworkListener(Thread):
	def __init__(self):
		super(NetworkListener, self).__init__()	# constructor thread
		self.gata_event = Event()
		self.gata_event.clear()
		# Adresa de server de la care am primit start game
		self.starter_addr = None 
		
	def set_window(self, window):
		self.window = window
		
	''' Add names to points '''
	def get_top(self, newtop):
		new_data = []
		for i in newtop:
			id, puncte = i
			nume = 'Unknown'
			for j in self.top_raw:
				idj, nj, pj = j
				if idj == id:
					nume = nj
					break
			new_data.append((id, nume, puncte))
		self.top_raw = new_data
		return self.top_raw
		
	def parse_message(self, data, addr):
		parse = data.split(":", 5)
		if len(parse) < 1:
			print "Parse error ", data
		elif parse[0] == 's':
			if self.starter_addr != None:
				print "Eroare: joc pornit deja"
				return
			self.starter_addr = addr
			if len(parse) < 4:
				print "Eroare pachet s: ", data
				sys.exit(10)			
			top_data = [(int(i.split(",")[0]),i.split(",")[1],0) for i in parse[4].split()]
			self.top_raw = top_data
			self.window.set_top(top_data)
			self.window.init_game((int(parse[1]), int(parse[2])))
		elif parse[0] == 'q':
			if self.starter_addr == None:
				print "Eroare: jocul nu a inceput"
				return
			if len(parse)<5:
				print "Eroare pachet q: ", data
				sys.exit(11)
			newtop = [(int(i.split(",")[0]),int(i.split(",")[1])) for i in parse[4].split()]
			self.window.set_top(self.get_top(newtop))
			if int(parse[1]) >= 0 & int(parse[2]) >= 0:
				self.window.set_question((int(parse[1]), int(parse[2])),parse[5])
			else:
				self.window.game_over()
		elif parse[0] == 'f':
			self.window.freeze()
		elif parse[0] == 'd':
			self.window.game_over()
		elif parse[0] == 'a':
			self.window.on_disconnect(None, None)
	
	def run(self):
		sUDP = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		sUDP.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		sUDP.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
		
		try:
			sUDP.bind((HOST, PORT))
		except socket.error, msg:
			print "Eroare la bind:", msg
			time.sleep(0.5)
			self.window.on_disconnect(None, None)
			self.window.error_entry.set_text("Cannot bind on udp")
			return
		print "Ascult pe udp dupa broadcast"
		
		gata = False
		while not self.gata_event.isSet():
			data, addr = sUDP.recvfrom(BUFFER_SIZE)
			if not data: break
			if (self.starter_addr != None) & (addr != self.starter_addr):
				print "Eroare: mesaj de la altcineva decat serverul. Ignor"
				continue
			# TODO: check addr so we don't have two servers at a time
			if data == 'quit':
				break
			self.parse_message(data, addr)			
		print "Oprit receptia udp."
		sUDP.close()
	
	def close(self):
		self.gata_event.set()

if __name__ == "__main__":
	n = NetworkListener()
	n.start()
	print "Started listening on udp %d." % PORT
